package geniusweb.protocol.session;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonSubTypes;

import geniusweb.protocol.NegoSettings;
import geniusweb.protocol.session.saop.SAOPSettings;
import geniusweb.references.PartyWithProfile;
import tudelft.utilities.logging.Reporter;

/**
 * interface for settings for session protocols. Immutable.
 */
@JsonSubTypes({ @JsonSubTypes.Type(value = SAOPSettings.class) })

public interface SessionSettings extends NegoSettings {

	/**
	 * Get participants
	 * 
	 * @return list of {@link PartyWithProfile} items.
	 */
	public List<PartyWithProfile> getParticipants();

	/**
	 * Allows modification of SessionSettings to include a party. This is needed for
	 * tournament auto-configuration of sessions.
	 * 
	 * @param partyprof the party with profile to be added
	 * @return new modified SessionSettings object
	 */
	public SessionSettings with(PartyWithProfile partyprof);

	/**
	 * @param logger the logger where the protocol can log events to.
	 * @return the an initialized and ready to use {@link SessionProtocol} that can
	 *         handle this Negotiation.
	 */
	@Override
	public SessionProtocol getProtocol(Reporter logger);

}
